@@ -6,7 +6,7 @@
 
 
 /*
- * $Id: filelog.h,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $
+ * $Id: filelog.h,v 1.2 2003/03/09 19:29:21 steveg Exp $
  */
 
 #ifndef __FILELOG_H
@@ -24,7 +24,7 @@
  */
 typedef enum { FL_CLOSED = 0, FL_OPEN, FL_SIZE, FL_ERROR } filelog_state_e ;
 
-struct filelog
+struct filelog_s
 {
 	int					fl_fd ;
 	filelog_state_e	fl_state ;
@@ -40,7 +40,7 @@ struct filelog
 #define FILELOG_DISABLE_SIZE_CONTROL( flp )	(flp)->fl_size_control = FALSE
 #define FILELOG_SIZE_CONTROL( flp )    		( (flp)->fl_size_control )
 
-#define FILELOG( xp )			((struct filelog *)xp->xl_data)
+#define FILELOG( xp )			((struct filelog_s *)xp->xl_data)
 
 #endif
 
